"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var JsDiff = require("diff");
var helper_1 = require("./helper");
var Generator = /** @class */ (function () {
    function Generator() {
    }
    /**
     * exec
     */
    Generator.prototype.exec = function (oldString, newString) {
        var output = [];
        var parts = JsDiff.diffChars(oldString, newString);
        for (var _i = 0, parts_1 = parts; _i < parts_1.length; _i++) {
            var part = parts_1[_i];
            var value = part.value;
            var prefix = part.added ? '<ins>' : part.removed ? '<del>' : '';
            var posfix = part.added ? '</ins>' : part.removed ? '</del>' : '';
            if (helper_1.Helper.isTitle(part)) {
                output.push(this.titleDiff(value, prefix, posfix));
            }
            else if (helper_1.Helper.isTable(part)) {
                output.push(this.tableDiff(value, prefix, posfix));
            }
            else if (helper_1.Helper.isList(part)) {
                output.push(this.listDiff(value, prefix, posfix));
            }
            else {
                output.push("" + prefix + value + posfix);
            }
        }
        return output.join('');
    };
    Generator.prototype.titleDiff = function (value, prefix, posfix) {
        var out = [];
        var match = Generator.titleRegexWithContent.exec(value);
        while (match !== null) {
            var spaces = match[1];
            var listOp = match[2];
            var afterOpSpaces = match[3];
            var content = match[4];
            out.push("" + spaces + listOp + afterOpSpaces + prefix + content + posfix);
            match = Generator.titleRegexWithContent.exec(value);
        }
        return out.join('\n');
    };
    Generator.prototype.listDiff = function (value, prefix, posfix) {
        var out = [];
        var match = Generator.listRegexWithContent.exec(value);
        while (match !== null) {
            var spaces = match[1];
            var listOp = match[2];
            var afterOpSpaces = match[3];
            var content = match[4];
            out.push("" + spaces + listOp + afterOpSpaces + prefix + content + posfix);
            match = Generator.listRegexWithContent.exec(value);
        }
        return out.join('\n');
    };
    Generator.prototype.tableDiff = function (value, prefix, posfix) {
        var out = [];
        var split = value.split('|');
        var startWithPipe = split[0].length === 0 ? '|' : '';
        var endsWithPipe = split[split.length - 1].length === 0 ? '|' : '';
        var filtered = split.filter(function (el) { return el.length !== 0; });
        for (var _i = 0, filtered_1 = filtered; _i < filtered_1.length; _i++) {
            var val = filtered_1[_i];
            out.push("" + prefix + val + posfix);
        }
        return startWithPipe + out.join('|') + endsWithPipe;
    };
    Generator.listRegexWithContent = /^([\r\n\t ]*)(\*|-|\+|\d+\.)([ ]*)(.*)$/gm;
    Generator.titleRegexWithContent = /^([\r\n\t ]*)(#+)([ ]*)(.*)$/gm;
    return Generator;
}());
exports.Generator = Generator;
//# sourceMappingURL=generator.js.map